/******************************************************************************

  Copyright(c) 2005 Intel Corporation. All rights reserved.

  This program is free software; you can redistribute it and/or modify it
  under the terms of version 2 of the GNU General Public License as
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  The full GNU General Public License is included in this distribution in the
  file called LICENSE.

  Contact Information:
  James P. Ketrenos <ipw2100-admin@linux.intel.com>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

******************************************************************************/
#include <linux/compiler.h>
#include <linux/config.h>
#include <linux/errno.h>
#include <linux/if_arp.h>
#include <linux/in6.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <linux/pci.h>
#include <linux/proc_fs.h>
#include <linux/skbuff.h>
#include <linux/slab.h>
#include <linux/tcp.h>
#include <linux/types.h>
#include <linux/version.h>
#include <linux/wireless.h>
#include <linux/etherdevice.h>
#include <asm/uaccess.h>

#include <net/ieee80211.h>


/*
 * NOTE:  The following tables are guidelines meant to improve the user
 * experience by being able to try and verify channels as being valid so that
 * error codes can be more readily provided if channel changes are requested
 * to those not supported by your hardware or geographic location.
 *
 * This code is not meant to be used to enforce regulatory compliance.
 *
 */
static const struct ieee80211_geo ieee80211_geos[IEEE80211_GEO_LAST] = {
	[IEEE80211_GEO_001] = { /* Canada, USA, Mexico */
		"001",
		.bg_channels = 11,
		.bg = { { 2412, 1, 0 }, { 2417, 2, 0 }, { 2422, 3, 0 },
			{ 2427, 4, 0 }, { 2432, 5, 0 }, { 2437, 6, 0 },
			{ 2442, 7, 0 }, { 2447, 8, 0 }, { 2452, 9, 0 },
			{ 2457,10, 0 }, { 2462,11, 0 } },
		.a_channels = 8,
		.a = { { 5180, 36, 0 }, { 5200, 40, 0 },
		       { 5220, 44, 0 }, { 5240, 48, 0 },
		       { 5260, 52, 0 }, { 5280, 56, 0 },
		       { 5300, 60, 0 }, { 5320, 64, 0 } },
	},
	[IEEE80211_GEO_002] = { /* Europe */
		"002",
		.bg_channels = 13,
		.bg = { { 2412, 1, 0 }, { 2417, 2, 0 }, { 2422, 3, 0 },
			{ 2427, 4, 0 }, { 2432, 5, 0 }, { 2437, 6, 0 },
			{ 2442, 7, 0 }, { 2447, 8, 0 }, { 2452, 9, 0 },
			{ 2457,10, 0 }, { 2462,11, 0 }, { 2467,12, 0 },
			{ 2472,13, 0 } },
		.a_channels = 12,
		.a = { { 5180, 36, 0 }, { 5200, 40, 0 },
		       { 5220, 44, 0 }, { 5240, 48, 0 },
		       { 5260, 52, 0 }, { 5280, 56, 0 },
		       { 5300, 60, 0 }, { 5320, 64, 0 },
		       { 5745,149, 0 }, { 5765,153, 0 },
		       { 5785,157, 0 }, { 5805,161, 0 } },
	},
	[IEEE80211_GEO_000] = { /* No geo -- attempt all channels */
		"000",
		.bg_channels = 14,
		.bg = { { 2412, 1, 0 }, { 2417, 2, 0 }, { 2422, 3, 0 },
			{ 2427, 4, 0 }, { 2432, 5, 0 }, { 2437, 6, 0 },
			{ 2442, 7, 0 }, { 2447, 8, 0 }, { 2452, 9, 0 },
			{ 2457,10, 0 }, { 2462,11, 0 }, { 2467,12, 0 },
			{ 2472,13, 0 }, { 2484,14, 0 } },
		.a_channels = 16,
		.a = { { 5170, 34, 0 }, { 5180, 36, 0 }, { 5190, 38, 0 },
		       { 5200, 40, 0 }, { 5210, 42, 0 }, { 5220, 44, 0 },
		       { 5230, 46, 0 }, { 5240, 48, 0 }, { 5260, 52, 0 },
		       { 5280, 56, 0 }, { 5300, 60, 0 }, { 5320, 64, 0 },
		       { 5745,149, 0 }, { 5765,153, 0 }, { 5785,157, 0 },
		       { 5805,161, 0 } },
	}
};

const struct ieee80211_geo *ieee80211_get_geo(struct ieee80211_device *ieee)
{
	if (unlikely(ieee->geography >= IEEE80211_GEO_LAST))
		BUG();

	return &ieee80211_geos[ieee->geography];
}

int ieee80211_is_valid_channel(struct ieee80211_device *ieee, u8 channel)
{
	const struct ieee80211_geo *geo = &ieee80211_geos[ieee->geography];
	int i;

	if (ieee->freq_band & IEEE80211_24GHZ_BAND)
		for (i = 0; i < geo->bg_channels; i++)
			if (geo->bg[i].channel == channel)
				return IEEE80211_24GHZ_BAND;

	if (ieee->freq_band & IEEE80211_52GHZ_BAND)
		for (i = 0; i < geo->a_channels; i++)
			if (geo->a[i].channel == channel)
				return IEEE80211_52GHZ_BAND;

	return 0;
}

u8 ieee80211_freq_to_channel(struct ieee80211_device *ieee, u32 freq)
{
	const struct ieee80211_geo *geo = &ieee80211_geos[ieee->geography];
	int i;

	freq /= 100000;

	if (ieee->freq_band & IEEE80211_24GHZ_BAND)
		for (i = 0; i < geo->bg_channels; i++)
			if (geo->bg[i].freq == freq)
				return geo->bg[i].channel;

	if (ieee->freq_band & IEEE80211_52GHZ_BAND)
		for (i = 0; i < geo->a_channels; i++)
			if (geo->a[i].freq == freq)
				return geo->a[i].channel;

	return 0;
}

EXPORT_SYMBOL(ieee80211_get_geo);
EXPORT_SYMBOL(ieee80211_is_valid_channel);
EXPORT_SYMBOL(ieee80211_freq_to_channel);
